import { createForm } from '@felte/solid';
import reporter from '@felte/reporter-tippy';
import { Button, Dialog, TextField } from '@kobalte/core';
import { Component, createSignal } from 'solid-js';
import 'tippy.js/dist/tippy.css';

interface CredentialsType {
  username: string;
  password: string;
  hostname?: string;
}

const [adminCredentials, setAdminCredentials] = createSignal<CredentialsType>();

export { adminCredentials };

interface Props {}

const Credentials: Component<Props> = (props) => {
  const validationHandler = async (values: any) => {
    const { database, username, password } = values;
    let errors: any = {};
    try {
      const response = await fetch(database, { mode: 'cors' });
      const dbStatus = await response.json();
      if (!dbStatus.couchdb) {
        errors.database = 'The URL is not a couchdb instance';
      }
    } catch (error) {
      // console.error({
      //   caller: 'Credentials / validationHandler / database root',
      //   error,
      // });
      errors.database = "Can't access the database";
    }
    if (!!errors.database) {
      return errors;
    }
    try {
      const authString = `${username}:${password}`;
      const headers = new Headers();
      headers.set('Authorization', 'Basic ' + btoa(authString));
      const response = await fetch(`${database}//_all_dbs`, {
        mode: 'cors',
        headers,
      });
      const dbStatus = await response.json();
      if (!!dbStatus.error) {
        errors.username = dbStatus.reason || 'Wrong username or password';
        errors.password = dbStatus.reason || 'Wrong username or password';
      }
      console.log({
        caller: 'Credentials / validationHandler',
        props,
        values,
        errors,
        dbStatus,
      });
      // if (!dbStatus.couchdb) {
      //   errors.database = 'The URL is not a couchdb instance';
      // }
    } catch (error) {
      console.error({
        caller: 'Credentials / validationHandler / database root',
        error,
      });
      errors.username = 'Wrong username or password';
      errors.password = 'Wrong username or password';
    }
    console.log({
      caller: 'Credentials / validationHandler',
      props,
      values,
      errors,
    });
    return errors;
  };

  const submitHandler = async (values: any, context: any) => {
    console.log({
      caller: 'Credentials / submitHandler',
      props,
      values,
      context,
    });
    setAdminCredentials(values);
  };

  let { form, data } = createForm({
    onSubmit: submitHandler,
    validate: validationHandler,
    extend: reporter(),
    //   initialValues: passwords().get(url),
  });

  return (
    <Dialog.Root isOpen={true}>
      <Dialog.Portal>
        <Dialog.Overlay class='dialog__overlay' />
        <div class='dialog__positioner'>
          <Dialog.Content class='dialog__content'>
            <div class='dialog__header'>
              <Dialog.Title class='dialog__title'>
                Admin user name and password
              </Dialog.Title>
              <Dialog.CloseButton class='dialog__close-button'>
                X
              </Dialog.CloseButton>
            </div>
            <Dialog.Description class='dialog__description'>
              <div>
                Please enter the database URL and its admin username and
                password.
              </div>
              <form use:form>
                <TextField.Root>
                  <TextField.Label>Database URL</TextField.Label>
                  <TextField.Input
                    type='url'
                    name='database'
                    required={true}
                    placeholder='Database URL'
                  />
                </TextField.Root>
                <TextField.Root>
                  <TextField.Label>User name</TextField.Label>
                  <TextField.Input
                    type='text'
                    name='username'
                    required={true}
                    placeholder='user name'
                  />
                </TextField.Root>
                <TextField.Root>
                  <TextField.Label>Password</TextField.Label>
                  <TextField.Input
                    type='text'
                    name='password'
                    required={true}
                    placeholder='Password'
                    autocomplete='off'
                  />
                  <TextField.ErrorMessage>
                    Please provide a valid password
                  </TextField.ErrorMessage>
                </TextField.Root>
                <Button.Root type='submit'>Save</Button.Root>
              </form>
            </Dialog.Description>
          </Dialog.Content>
        </div>
      </Dialog.Portal>
    </Dialog.Root>
  );
};

export default Credentials;
